@@ -1,6 +1,6 @@
 ;;; ledger-commodities.el --- Helper code for use with the "ledger" command-line tool
 
-;; Copyright (C) 2003-2014 John Wiegley (johnw AT gnu DOT org)
+;; Copyright (C) 2003-2016 John Wiegley (johnw AT gnu DOT org)
 
 ;; This file is not part of GNU Emacs.
 
@@ -33,10 +33,15 @@
   :type 'string
   :group 'ledger-reconcile)
 
-(defcustom ledger-scale  10000
-  "The 10 ^ maximum number of digits you would expect to appear in your reports.
-This is a cheap way of getting around floating point silliness in subtraction"
-  :group 'ledger)
+(defun ledger-read-commodity-with-prompt (prompt)
+  "Read commodity name after PROMPT.
+
+Default value is `ledger-reconcile-default-commodity'."
+  (let* ((buffer (current-buffer))
+         (commodities (with-temp-buffer
+                        (ledger-exec-ledger buffer (current-buffer) "commodities")
+                        (split-string (buffer-string) "\n" t))))
+    (completing-read prompt commodities nil t nil nil ledger-reconcile-default-commodity)))
 
 (defun ledger-split-commodity-string (str)
   "Split a commoditized string, STR, into two parts.
@@ -86,11 +91,7 @@ Returns a list with (value commodity)."
 (defun -commodity (c1 c2)
   "Subtract C2 from C1, ensuring their commodities match."
   (if (string= (cadr c1) (cadr c2))
-                                        ; the scaling below is to get around inexact
-                                        ; subtraction results where, for example 1.23
-                                        ; - 4.56 = -3.3299999999999996 instead of
-                                        ; -3.33
-      (list (/ (-  (* ledger-scale (car c1)) (* ledger-scale (car c2))) ledger-scale) (cadr c1))
+      (list (-(car c1) (car c2)) (cadr c1))
     (error "Can't subtract different commodities %S from %S" c2 c1)))
 
 (defun +commodity (c1 c2)
@@ -100,22 +101,21 @@ Returns a list with (value commodity)."
     (error "Can't add different commodities, %S to %S" c1 c2)))
 
 (defun ledger-strip (str char)
-  (let (new-str)
-    (concat (dolist (ch (append str nil) new-str)
-              (unless (= ch char)
-                (setq new-str (append new-str (list ch))))))))
+  "Return STR with CHAR removed."
+  (replace-regexp-in-string char "" str))
 
 (defun ledger-string-to-number (str &optional decimal-comma)
   "improve builtin string-to-number by handling internationalization, and return nil if number can't be parsed"
   (let ((nstr (if (or decimal-comma
                       (assoc "decimal-comma" ledger-environment-alist))
-                  (ledger-strip str ?.)
-                (ledger-strip str ?,))))
+                  (ledger-strip str ".")
+                (ledger-strip str ","))))
     (while (string-match "," nstr)  ;if there is a comma now, it is a thousands separator
       (setq nstr (replace-match "." nil nil nstr)))
     (string-to-number nstr)))
 
 (defun ledger-number-to-string (n &optional decimal-comma)
+  "number-to-string that handles comma as decimal."
   (let ((str (number-to-string n)))
     (when (or decimal-comma
               (assoc "decimal-comma" ledger-environment-alist))
@@ -134,6 +134,7 @@ longer ones are after the value."
       (concat commodity " " str))))
 
 (defun ledger-read-commodity-string (prompt)
+  "Read an amount from mini-buffer using PROMPT."
   (let ((str (read-from-minibuffer
               (concat prompt " (" ledger-reconcile-default-commodity "): ")))
         comm)
