@@ -1,6 +1,6 @@
 ;;; ledger-xact.el --- Helper code for use with the "ledger" command-line tool
 
-;; Copyright (C) 2003-2014 John Wiegley (johnw AT gnu DOT org)
+;; Copyright (C) 2003-2016 John Wiegley (johnw AT gnu DOT org)
 
 ;; This file is not part of GNU Emacs.
 
@@ -25,6 +25,11 @@
 
 ;;; Code:
 
+(require 'eshell)
+(require 'ledger-regex)
+(require 'ledger-navigate)
+;; TODO: This file depends on code in ledger-mode.el, which depends on this.
+
 (defcustom ledger-highlight-xact-under-point t
   "If t highlight xact under point."
   :type 'boolean
@@ -42,7 +47,7 @@
 (defun ledger-highlight-xact-under-point ()
   "Move the highlight overlay to the current transaction."
   (if ledger-highlight-xact-under-point
-      (let ((exts (ledger-navigate-find-xact-extents (point)))
+      (let ((exts (ledger-navigate-find-element-extents (point)))
             (ovl ledger-xact-highlight-overlay))
         (if (not ledger-xact-highlight-overlay)
             (setq ovl
@@ -52,7 +57,7 @@
                                       (current-buffer) t nil)))
           (move-overlay ovl (car exts) (cadr exts)))
         (overlay-put ovl 'face 'ledger-font-xact-highlight-face)
-        (overlay-put ovl 'priority 100))))
+        (overlay-put ovl 'priority '(nil . 99)))))
 
 (defun ledger-xact-payee ()
   "Return the payee of the transaction containing point or nil."
@@ -84,9 +89,8 @@ MOMENT is an encoded date"
     (when (and (eobp) last-xact-start)
       (let ((end (cadr (ledger-navigate-find-xact-extents last-xact-start))))
         (goto-char end)
-        (if (eobp)
-            (insert "\n")
-          (forward-line))))))
+        (insert "\n")
+        (forward-line)))))
 
 (defun ledger-xact-iterate-transactions (callback)
   "Iterate through each transaction call CALLBACK for each."
@@ -134,6 +138,7 @@ MOMENT is an encoded date"
                            (string-to-number (match-string 2 date)))))
     (ledger-xact-find-slot encoded-date)
     (insert transaction "\n")
+    (beginning-of-line -1)
     (ledger-navigate-beginning-of-xact)
     (re-search-forward ledger-iso-date-regexp)
     (replace-match date)
@@ -142,7 +147,7 @@ MOMENT is an encoded date"
         (goto-char (match-beginning 0)))))
 
 (defun ledger-delete-current-transaction (pos)
-  "Delete the transaction surrounging point."
+  "Delete the transaction surrounging POS."
   (interactive "d")
   (let ((bounds (ledger-navigate-find-xact-extents pos)))
     (delete-region (car bounds) (cadr bounds))))
@@ -179,8 +184,8 @@ correct chronological place in the buffer."
              (goto-char (point-min))
              (if (looking-at "Error: ")
                  (error (concat "Error in ledger-add-transaction: " (buffer-string)))
-							 (ledger-post-align-postings (point-min) (point-max))
-							 (buffer-string)))
+               (ledger-post-align-postings (point-min) (point-max))
+               (buffer-string)))
            "\n"))
       (progn
         (insert (car args) " \n\n")
