@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2014, John Wiegley.  All rights reserved.
+ * Copyright (c) 2003-2016, John Wiegley.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -243,6 +243,9 @@ namespace {
 
 void export_commodity()
 {
+#if BOOST_VERSION >= 106000
+  python::register_ptr_to_python< shared_ptr<commodity_pool_t> >();
+#endif
   class_< commodity_pool_t, shared_ptr<commodity_pool_t>,
           boost::noncopyable > ("CommodityPool", no_init)
     .add_property("null_commodity",
